package com.simuwang.base.pojo.dto.report;

import cn.hutool.core.collection.ListUtil;
import com.simuwang.base.common.conts.Constants;

import java.io.Serial;
import java.io.Serializable;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

/**
 * @author wangzaijun
 * @date 2024/10/12 11:00
 * @description 自定义简单表格对象（excel工作表中的多个表格解析）
 */
public class SimpleTable implements Serializable {
    @Serial
    private static final long serialVersionUID = Constants.DEFAULT_SERIAL_ID;
    private final String tableKey;
    /**
     * 表格标题
     */
    private final String title;
    /**
     * 表格数据行
     */
    private final List<List<String>> tables;
    /**
     * 表格列数
     */
    private int colCount;
    /**
     * 表格行数
     */
    private int rowCount;

    public SimpleTable(String tableKey, String title, int colCount) {
        this.tableKey = tableKey;
        this.title = title;
        this.colCount = colCount;
        this.tables = ListUtil.list(true);
    }

    public void addRow(List<String> row) {
        tables.add(row);
        this.rowCount++;
        if (this.colCount == 0) {
            this.colCount = this.tables.stream().map(List::size).max(Comparator.naturalOrder()).orElse(0);
        }
    }

    public int getColCount() {
        return colCount;
    }

    public int getRowCount() {
        return rowCount;
    }

    public String getTableKey() {
        return tableKey;
    }

    public String getTitle() {
        return title;
    }

    public List<List<String>> getTables() {
        int subRows = this.rowCount - this.tables.size();
        if (subRows <= 0) {
            return this.tables;
        }
        for (int i = 0; i < subRows; i++) {
            List<String> row = ListUtil.list(true);
            for (int j = 0; j < this.colCount; j++) {
                row.add(null);
            }
            this.tables.add(row);
        }
        return this.tables;
    }

    /**
     * 实现迭代器接口，支持按行遍历表格数据
     *
     * @return /
     */
    public Iterator<List<String>> iterator() {
        return new TableIterator(this.getTables());
    }

    /**
     * 获取某个单元格的数据
     *
     * @param row    行号
     * @param column 列号
     * @return 单元格内容
     */
    public String getCell(int row, int column) {
        List<List<String>> rows = this.getTables();
        if (row < 0 || row >= rows.size() || column < 0 || column >= rows.get(row).size()) {
            throw new IndexOutOfBoundsException("Invalid row or column index");
        }
        return rows.get(row).get(column);
    }

    @Override
    public String toString() {
        return "SimpleTable{" +
                "title='" + title + '\'' +
                ", tables=" + tables +
                '}';
    }

    /**
     * 内部迭代器类
     */
    private static class TableIterator implements Iterator<List<String>> {
        private final List<List<String>> tables;
        private int currentIndex = 0;

        public TableIterator(List<List<String>> tables) {
            this.tables = tables;
        }

        @Override
        public boolean hasNext() {
            return currentIndex < tables.size();
        }

        @Override
        public List<String> next() {
            return tables.get(currentIndex++);
        }
    }
}
